# File name: ColourMe.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 30 October 2019
# Date last modified: 30 October 2019
# MicroPython Version: 1.10 for the Kookaberry V4-05 from 2019-08-08 firmware update
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Demonstrates a colour whell using NeoPixels for display and the Kookaberry's tilt as control
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: P2 for NeoPixels
# /lib files: neopixel.py
# /root files: Nil
# Other dependencies: NeoPixel firmware update 2019-08-08
# Complementary apps:
#------------------------------------------

# Begin code
import time, kooka, machine, math, fonts, neopixel

disp = kooka.display

# Routine to draw an arc specified in degrees
def drawarc(x,y,r,arcstart,arcstop): 
    arcinc = 5    # increment of arc to be draw in degrees
    arclen = abs(arcstop-arcstart)
    lines = int(arclen/arcinc)
    for i in range(0,lines):
        angle1 = (arcstart + i*arcinc)/180 * math.pi
        angle2 = (arcstart + (i+1)*arcinc)/180 * math.pi
        x1 = x + int(r * math.sin(angle1))
        y1 = y - int(r * math.cos(angle1))
        x2 = x + int(r * math.sin(angle2))
        y2 = y - int(r * math.cos(angle2))
        disp.line(x1, y1, x2 , y2, 1)
#        print(x1, y1, x2 , y2, 1)
    return


# Initial conditions
compass_x = 80    # centre of compass dial
compass_y = 32    
compass_radius = 22    # radius of compass dial
compass_points = 6    # number of points on the compass dial
compass_ticks = 0    # radius of the points on the dial
compass_txt = ['R','Y','G','C','B','V']
max_intensity = 64    # limit upper intensity of LEDs
min_intensity = 15    # switch off LEDs below this level
pixels_in_array = 4
black = (0,0,0)
tilt_sensitivity = 2

# Set up objects
np = neopixel.NeoPixel(machine.Pin('P2'), pixels_in_array)
np.fill(black)
np.write()

# Main loop begins here
while not kooka.button_a.was_pressed():
# Display the static text
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text("ColourMe", 0, 6)
    disp.text('A-exit', 0, 60)
    drawarc(compass_x,compass_y,compass_radius,0,360)
    drawarc(compass_x,compass_y,compass_radius*0.5,0,360)
# Draw the RGB colour points
    disp.setfont(fonts.mono6x7)
    for b in range(0, compass_points):
        a = 2 * math.pi / compass_points * b
        a1x = int(compass_radius * math.sin(a)) + compass_x
        a1y = int(compass_radius * math.cos(a)) + compass_y
        a2x = int(compass_ticks * math.sin(a)) + compass_x
        a2y = int(compass_ticks * math.cos(a)) + compass_y
        disp.line(a1x,a1y,a2x,a2y,1)
        a1x = int((compass_radius+5) * math.sin(a)) + compass_x - 3
        a1y = int((compass_radius+5) * math.cos(a)) + compass_y + 3
        disp.text('%s' % compass_txt[b], a1x, a1y)


# Read inputs
# Obtain and display Kookaberry orientation to get level
    x, y, z = kooka.accel.get_xyz()
    x = min(1, max(-1, x / 9.8 * tilt_sensitivity))
    y = min(1, max(-1, y / 9.8 * tilt_sensitivity))
    u = compass_x - int(y * compass_radius)
    v = compass_y - int(x * compass_radius)
    level = abs(x) <= 0.1 and abs(y) <= 0.1
    disp.line(u - 5, v, u + 5, v, 1)
    disp.line(u, v - 5, u, v + 5, 1)
    if level: kooka.led_green.on()
    else: kooka.led_green.off()
# Perform calculations
    xr = max(0,(0.33 - 0.67*x))    # range tilt over 2/3 of diameter
    red_level = int(xr * max_intensity)    # scale to intensity level
    angle = 2*math.pi/3    # perform coordinate rotation to 120deg
    yg = x * math.sin(angle) + y * math.cos(angle)
    yg = max(0,(0.33 + 0.67*yg))    # repeat tilt and intensity calc for green and blue
    green_level = int(yg * max_intensity)
    angle = -2*math.pi/3
    yb = x * math.sin(angle) + y * math.cos(angle)
    yb = max(0,(0.33 - 0.67*yb))
    blue_level = int(yb * max_intensity)
    # limit lower bound of intensity so LEDs switch off cleanly because accelerometer is imperfect too
    if red_level < min_intensity: red_level = 0
    if green_level < min_intensity: green_level = 0
    if blue_level < min_intensity: blue_level = 0
    # work out colour proportions as a precentage
    sum_level = red_level + green_level + blue_level
    red_comp = int(red_level / sum_level * 100) 
    green_comp = int(green_level / sum_level * 100) 
    blue_comp = int(blue_level / sum_level * 100) 
    
# Display variables
    disp.setfont(fonts.mono6x7)
    disp.text('R%3d%%' % red_comp, 0,20)
    disp.text('G%3d%%' % green_comp, 0,30)
    disp.text('B%3d%%' % blue_comp, 0,40)

# Set outputs
    np[0] = (red_level,0,0)    # show red intensity
    np[1] = (0,green_level,0)    # show green intensity
    np[2] = (0,0,blue_level)    # show blue intensity
    np[3] = (red_level,green_level,blue_level)    # show combined colour
    np.write()

# Adjust timers and counters
    disp.show()
# Clean up and exit
np.fill(black)
np.write()
kooka.led_green.off()