# File name: 
__name__ = 'GameOfLife'
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 25 June 2020
# Date last modified: 26 June 2020
# MicroPython Version: 1.12 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Implements Conway's Life simulation on the Kookaberry display
# The rules are:
# 1. Any live cell (pixel) with two or three live neighbours survives.
# 2. Any dead cell with three live neighbours becomes a live cell.
# 3. All other live cells die in the next generation. Similarly, all other dead cells stay dead.
# The cell (pixel) field needs to be seeded and is done so at random
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: Nil
# /lib files: Nil
# /root files: Nil
# Other dependencies: Nil
# Complementary apps: Nil
#------------------------------------------
# Begin code
import machine, kooka, fonts, random
disp = kooka.display
# The initial population
pop_seed = 500
# The position and size of the Life game field
width = 80
height = 40
x_ctr = 40
y_ctr = 30
x_min = x_ctr - int(width/2)
x_max = x_ctr + int(width/2)
y_min = y_ctr - int(height/2)
y_max = y_ctr + int(height/2)
space = (width-4) * (height-4) # population space - allows for borders of field

# Function determines a cell's value from the surrounding cells
def cell(x, y, framebuf):
    cell_count = 0
    cell_value = framebuf.pixel(x, y)
    for j in range(-1,2):
        for i in range(-1,2):
            if i == 0 and j == 0: pass 
            else: cell_count += framebuf.pixel(x+i, y+j)
            if cell_count > 3: break    # we have reached maximum of interest
#            print(x+i,y+j,cell_count)
    if cell_count == 3: cell_new = 1
    elif cell_count == 2 and cell_value: cell_new = 1
    else: cell_new = 0
    framebuf.pixel(x, y, cell_new)
    return (cell_new - cell_value)    # change in cell count
#    print(x,y,cell_value,cell_count)

# Set up a datalogging file
fname = __name__ + '.csv'
f = open(fname, 'w+')
f.write('Generation,Population,Births,Deaths,Growth-%,Density-%\n')
f.close()

# Draw the static parts of the display
disp.fill(0)
disp.setfont(fonts.mono6x7)
disp.text('%s' % __name__, 0, 6)
disp.text('A-x C- 25% D+', 0, 60)
disp.rect(x_min, y_min, (x_max - x_min), (y_max - y_min), 1) # field boundary
# seed the field with cells
for i in range(0,pop_seed):    
    x = random.randint(x_min + 2,x_max - 3)
    y = random.randint(y_min + 2,y_max - 3)
    disp.pixel(x, y, 1)
disp.show()
# do an initial census
population = 0
for x in range(x_min+2, x_max-2):    # range over the field
    for y in range(y_min+2, y_max-2):
        population += disp.pixel(x, y)    # count the cell if alive
pop_min = population
pop_max = population
generation = 0
# The main loop
while not kooka.button_a.was_pressed(): # quit if button A pressed
    births = 0
    deaths = 0
    # cut population by 25%
    if kooka.button_c.was_pressed():
        target = int(population / 4)
        count = 0
        while count < target:
            x = random.randint(x_min + 2,x_max - 3)
            y = random.randint(y_min + 2,y_max - 3)
            if disp.pixel(x, y):
                disp.pixel(x, y, 0)
                population -= 1
                deaths += 1
                count += 1
        print('Cut:',deaths)
    # boost population by 25%
    if kooka.button_d.was_pressed():
        target = int(population / 4)
        count = 0
        while count < target:
            x = random.randint(x_min + 2,x_max - 3)
            y = random.randint(y_min + 2,y_max - 3)
            if not disp.pixel(x, y):
                disp.pixel(x, y, 1)
                population += 1
                births += 1
                count += 1
        print('Boost:',births)
    # Apply Game of Life rules to the field
    for x in range(x_min+2, x_max-2):    # range over the field
        for y in range(y_min+2, y_max-2):
            change = cell(x, y, disp)    # compute cell life or death
            population += change
            if change == 1: births += 1
            if change == -1: deaths += 1
        disp.show()
    pop_max = max(pop_max, population)
    pop_min = min(pop_min, population)
    density = int(population / space * 100)
    growth = int((births - deaths) / max(population, 1) * 100)
    generation += 1
    disp.fill_rect(81,0,50,63,0)    # clear the statistics counter display area
#    disp.text('G:%d Pop:%d:%d:%d D:%0.2f' % (generation,pop_min,population,pop_max,density), 24,60)    # display cell count
    disp.setfont(fonts.mono6x7)
    disp.text('Gen:%d' % generation, 82, 6)
    disp.text('P:%d' % population, 82, 15)
    disp.text('Gth:%d%%' % growth, 82, 24)
    disp.text('D:%d%%' % density, 82, 33)
    disp.setfont(fonts.mono5x5)
    disp.text('Bths:%d' % births, 82, 40)
    disp.text('Dths:%d' % deaths, 82, 47)
    disp.text('Pmin:%d' % pop_min, 82, 54)
    disp.text('Pmax:%d' % pop_max, 82, 61)
    disp.show()    # update the display every cell
    
    # Update the data logging file
    f = open(fname, 'a+')
    f.write('%d,%d,%d,%d,%d,%d\n' % (generation,population,births,deaths,growth,density))
    f.close()
