# File name: KookatimeXX.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 30 July 2018
# Date last modified: 31 December 2018
# MicroPython Version: 1.9.4 for the Kookaberry
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Program to read a time tuple from from an external RTC and a file and update the real time clock in the Kookaberry
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: DS3231 battery real time clock on port P3 (optional)
# /lib files: Timeconfig.py, doomsday.py, Kapputils.py
# /root files: KappConfig.txt
# Other dependencies: Nil
# Complementary apps: KookaTimeRx.py
#------------------------------------------

import TimeConfig, doomsday    # timekeeping utility program
import os, time, json, machine, kooka, fonts
from Kapputils import config    # utility to read the configuration file
    
disp = kooka.display    # instantiate the display
rtc = machine.RTC()    # instantiate the in-built real time clock
params = config('Kappconfig.txt')   # read the configuration file
# set up the radio for later use
kooka.radio.enable()
chan = int(params['CHANNEL'])      # use channel from the configuration file
baud = int(params['BAUD'])      # use data rate from the configuration file
pwr = int(params['POWER'])      # use transmit power from the configuration file

kooka.radio.config(channel=chan, data_rate=baud, power=pwr) # set up the radio

ftime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
ktime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
months = ["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"]
updptr = [3,2,1,0,4,5,6]    # order of updating the RTC elements
updmin = [1,1,1,2015,0,0,0] # minimums for each RTC element
updmax = [7,31,12,2040,23,59,59]    # maximums for each RTC element
cursor = [20,30,30,30,40,40,40]     # vertical cursor positions
cursorx = [0,0,28,60,0,28,52]    # horizontal cursor positions

# Display the clock and facilitate manual correction
p_upd = 7   # initialise the update pointer
while not kooka.button_a.was_pressed():
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('Real Time Clock', 0, 6)
    disp.setfont(fonts.mono6x7)
    if p_upd >= 7: disp.text('A-exit  Tx  Sav  Next', 0, 60)
    else: disp.text('A-exit   -   +  Next', 0, 60)
    ktime = rtc.datetime()  # Refresh the time
    for i in range(0,7): ftime[i] = ktime[i]
    disp.setfont(fonts.mono8x8)
    disp.text('%s' % doomsday.calc_dow(ftime[2],ftime[1],ftime[0]), 12, 20)    # Display Weekday
    disp.text('%2d-%s-%d' % (ftime[2], months[ftime[1]-1], ftime[0]),12, 30)   #Display date
    disp.text('%02d:%02d:%02d' % (ftime[4], ftime[5], ftime[6]),12, 40)   #Display time
    if p_upd < 7 : disp.text('>', cursorx[p_upd], cursor[p_upd])    # cursor to show what we are adjusting
    disp.show()
    dirty = 0   # Flag indicates a field was modified
    if kooka.button_d.was_pressed(): 
        if p_upd == 7: TimeConfig.putTime()
        else: 
            ftime[updptr[p_upd]] += 1
            if ftime[updptr[p_upd]] > updmax[p_upd]: ftime[updptr[p_upd]] = updmin[p_upd]    # roll over to minimum value
            dirty = 1
    if kooka.button_c.was_pressed(): 
        if p_upd == 7: 
            kooka.radio.send('[%4d,%2d,%2d,%2d,%2d,%2d]' % (ftime[0],ftime[1],ftime[2],ftime[4],ftime[5],ftime[6]))
        else: 
            ftime[updptr[p_upd]] -= 1
            if ftime[updptr[p_upd]] < updmin[p_upd]: ftime[updptr[p_upd]] = updmax[p_upd]
            dirty = 1
    if kooka.button_b.was_pressed(): 
        p_upd += 1
        if p_upd > 7: p_upd = 1    # Avoid adjusting day of week which is automatic
    if dirty : 
        ftime[3] = doomsday.dow_index(ftime[2],ftime[1],ftime[0])    # update day of week
        if ftime[3] == 0: ftime[3] = 7    # adjust Sunday to match Kooka's clock format
        rtc.datetime(tuple((ftime[0],ftime[1],ftime[2],ftime[3],ftime[4],ftime[5],ftime[6],0))) # Update the RTC
    time.sleep_ms(100)
# Clean up and exit
TimeConfig.putTime()    # save the time in the file and the external RTC
kooka.radio.disable()