# File name: SenseRx.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 17 October 2019
# Date last modified: 26 February 2020
# MicroPython Version: 1.12 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# This program listens to the packet radio, and displays messages received.
# The program records what it receives to file in SenseRxLog.csv in the root directory
# It is intended to be used with the SenseXXX apps that measure and log various sensors
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: Nil
# /lib files: Kapputils.py, kcpu.py
# /root files: Kappconfig.txt
# Other dependencies: Nil
# Complementary apps: SenseXXX.py sensor apps
#------------------------------------------
# Begin code
import TimeConfig
import machine, kooka, time, json, fonts
import kcpu    # module checks on battery charge
import doomsday    # module converts date to day of week
from Kapputils import config    # utility to read the configuration file
disp_lines = ['']*5
disp_length = len(disp_lines) - 1
disp_locs = [15,24,33,42,51]

# Use the Kookaberry Real Time Clock - user will have to set the time beforehand.
ktime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
rtime = [0]*6   # Kookaberry time tuple [YYYY,MM,DD,HH,MM,SS]
ftime = [0]*8   # Kookaberry time tuple [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
rtc = machine.RTC()    # instantiate the Real Time Clock
time_sent = False    # one shot flag for sending time updates

params = config('Kappconfig.txt')   # read the configuration file
# set up the radio for later use
kooka.radio.enable()
chan = int(params['CHANNEL'])      # use channel from the configuration file
baud = int(params['BAUD'])      # use data rate from the configuration file
pwr = int(params['POWER'])      # use transmit power from the configuration file
kooka.radio.config(channel=chan, data_rate=baud, power=pwr, length=40) # set up the radio

# Prepare the logging file
fname = 'SenseRxLog.csv'
f = open(fname,'w+')    # open file for writing - overwrites any prior file
f.write('Date-Time,ID,Sensor,Data\n')
disp = kooka.display    # initialise the display
msg_count = 0

while not kooka.button_a.was_pressed()and kcpu.battery_ok():
    # Construct the time string
    ktime = rtc.datetime()  # Refresh the time
    timestr = "%02d-%02d-%04d-%02d:%02d:%02d" % (ktime[2],ktime[1],ktime[0],ktime[4],ktime[5],ktime[6])    # Kookaberry time tuple is [YYYY,MM,DD,WD,HH,MM,SS,SUBS]
    if ktime[6]%10 == 0 and not time_sent: 
        msgstr = '[%4d,%2d,%2d,%2d,%2d,%2d]' % (ktime[0],ktime[1],ktime[2],ktime[4],ktime[5],ktime[6])
        kooka.radio.send(msgstr)    # send time every 10 seconds
        time_sent = True
        print(msgstr)
    elif ktime[6]%10 > 0: time_sent = False    # resets time update flag
    
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('SenseRx', 0, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('A-exit', 6, 62)	   # A Key = Exit
    disp.setfont(fonts.mono5x5)
#    disp.text('Msg: %d' % msg_count, 50, 62)	   # No of messages received
    for i,line in enumerate(disp_lines):
        disp.text('%s' % line, 4, disp_locs[i])    # display last received messages
    msg = kooka.radio.receive()    #listen for a message
    if msg:    # radio has received a message
        print(msg)
        if len(msg) == 21 and msg[1].isdigit() and msg[2].isdigit() and msg[3].isdigit() and msg[4].isdigit():  pass  # is a time update so ignore
        else:    # not a time update so process
            msg_count += 1
            disp_lines[0] = disp_lines[1]
            disp_lines[1] = disp_lines[2]
            disp_lines[2] = disp_lines[3]
            disp_lines[3] = disp_lines[4]
            sensordata = json.loads(msg)
            sensorstr = ''
            for i,data in enumerate(sensordata):
                sensorstr = sensorstr + ',' + data
#                print(sensorstr)
            disp_lines[disp_length] = '%3d' % msg_count + ' ' + sensorstr
            f.write('%s%s\n' % (timestr,sensorstr))    # update the log file
    disp.show()
    
# Clean up and exit
kooka.radio.disable()    # turn packet radio off
f.close()    # safely close the log file

