# File name: 
__name__ = 'SpeedTrap'
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 2 June 2020
# Date last modified: 2 July 2020
# MicroPython Version: 1.12 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# This app measures the speed of an object passing over two trigger points.
# The trigger points can be switches, or magnetic or light sensors
# Provision is made for triggering on the leading or trailing edge of the trigger
# That is, a contact closure, or opening such as a light beam interruption
#------------------------------------------
# Dependencies:
# I/O ports and peripherals: digital sensors on P1 and P2
# /lib files: kbtimer, Kapputils
# /root files: nil
# Other dependencies: nil
# Complementary apps: none
#------------------------------------------

# Begin code
import machine, kooka, fonts, time, ujson
import kbtimer
from Kapputils import config    # module to read the configuration file

inputA = machine.Pin('P1', machine.Pin.IN, machine.Pin.PULL_DOWN)    # create digital input for button or other digital device
inputB = machine.Pin('P2', machine.Pin.IN, machine.Pin.PULL_DOWN)    # create digital input for button or other digital device
b_timer = kbtimer.KBTimer(kooka.button_b, 1000) # instantiate the B buttom timer object
# Set up the radio
params = config('Kappconfig.txt')   # read the configuration file
# set up the radio for later use
kooka.radio.enable()
chan = int(params['CHANNEL'])      # use channel from the configuration file
baud = int(params['BAUD'])      # use data rate from the configuration file
pwr = int(params['POWER'])      # use transmit power from the configuration file
kooka.radio.config(channel=chan, data_rate=baud, power=pwr, length=40) # set up the radio
id=''
for i in range(0,min(2,len(params['NAME']))): id = id + params['NAME'][i]
for i in range(0,min(2,len(params['SURNAME']))): id = id + params['SURNAME'][i]
sndmsg = [id,'Spd','R','S']    # Structure of radio msg

# Initial conditions
inputs = [0,0]    # mirrors the detector input states
levels = [True,True]    # detector logic - 1 = leading edge, 0 = trailing edge
triggers = [0,0]    #whether a gate has triggered
dets = [0,0]    # detection events - when a detector was last triggered
state = 3    # 0=waitng, 1=triggered, 2=measured, 3=startup, 4=configuration
distance = 10 # inter-detector distance in centimetres
run = 0    # counter of the number of timing runs
timer = 0    # elapsed time between detections
speed = 0    # the last calculated speed
blink = 1    # used for blinking parts of the display
blink_time = 500    # half the blinking period
t_blink = time.ticks_ms()
# Prepare the logging file
fname = __name__+'.csv'    # the file name
f = open(fname,'w+')         # open a text file for writing
f.write('Run,Distance-cm,Time-secs,Speed-cm/s\n')   # write the heading line
f.close()

# Set up objects
disp = kooka.display    # initialise the OLED display

while not kooka.button_a.was_pressed():
# Update and toggle the blinker
    if time.ticks_diff(time.ticks_ms(), t_blink) >= 0:
        blink = not blink    # toggle the blinker
        t_blink += blink_time    # prepare for next timeout
# Adjust the input logic using the C and D keys
    if state == 2 or state == 3:
        if kooka.button_c.was_pressed(): levels[0] = not levels[0]
        if kooka.button_d.was_pressed(): levels[1] = not levels[1]
# Capture detection events
    a = inputA.value()
    b = inputB.value()
    inputs[0] = (a and levels[0]) or (not a and not levels[0])
    inputs[1] = (b and levels[1]) or (not b and not levels[1])
    for i in range(0,2):
        triggers[i] = inputs[i] and not triggers[i]    # detect a 0 to 1 transition
    if state == 0 and triggers[0]:    # Waiting for the first detection
        dets[0] = time.ticks_ms()    # set the current time counter
        state = 1    # set to waiting for second detection
    elif state == 1 and triggers[1]:    # Waiting for the second detection
        dets[1] = time.ticks_ms()    # set the current time counter
        timer = time.ticks_diff(dets[1], dets[0])/1000    # in milliseconds
        speed = distance / timer # in cm/sec
        f = open(fname,'a+')         # open a text file for writing
        f.write('%d,%d,%0.3f,%.2f\n' % (run,distance,timer,speed))    # record the results
        f.close()
        sndmsg[2] = '%d' % run
        sndmsg[3] = '%.2f' % speed
        kooka.radio.send(ujson.dumps(sndmsg))    # send the radio message
        print(sndmsg)
        state = 2    # set to waiting for second detection
    else: pass    # just keep the time and speed displayed
    if state == 1:    # update the timer
        timer = time.ticks_diff(time.ticks_ms(),dets[0])/1000    # in milliseconds
    if kooka.button_b.was_pressed():    # reset the gates
        if state >= 2: run += 1    # increment run only after run finish
        dets = [0,0]
        trigs = [0,0]
        timer = 0
        speed = 0
        state = 0

    if b_timer.isexpired():
        state = 4    # put the app into configuration state
        if run > 0: run -= 1    # compensate out additional run counted
        timer = 0
        b_timer.reset()      # reset the button timer
# Adjust the input logic using the C and D keys
    if state == 4:    # adjust the inter-sensor distance
        if kooka.button_c.was_pressed(): distance -= 1
        if kooka.button_d.was_pressed(): distance += 1
        distance = max(10, distance)    # limite the minimum distance
# Prepare the display
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('%s' % __name__, 0, 6)    # title of the app
    disp.setfont(fonts.mono5x5)
    disp.text('%dcm' % distance, 76,6)
    disp.setfont(fonts.mono6x7)
    disp.text('R:%d' % run, 100,6)
    if state == 4:
        disp.text('--Adj-Dist--',27,50)
    else:
        disp.text('---Levels---',27,50)
    disp.text('---Inputs---',27,40)
    if state== 4:
        disp.text('A:x  C:-   D:+   B:Rst', 0, 60)
    else:
        disp.text('A:x  C:L1  D:L2 B:Rst', 0, 60)    # button assignments
# Draw and number the timing gates
    for i in range(0,2):
        disp.rect(10+i*100,12,10,20,1)    # draw hollow rectangles for gates
    if state == 1 or state == 2: disp.fill_rect(10,12,10,20,1) # first gate triggered
    if state == 2: disp.fill_rect(110,12,10,20,1) # second gate triggered
    b1 = 1
    if state == 0: b1 = blink
    b2 = 1
    if state == 1: b2 = blink
    disp.text('1', 12,25,b1)
    disp.text('2', 112,25,b2)
    disp.text('%d' % a, 12,40)
    disp.text('%d' % b, 112,40)
    disp.text('%d' % levels[0], 12,50)
    disp.text('%d' % levels[1], 112,50)
    disp.text('T: %.2fs' % timer, 28, 20)    # display the timer
    disp.text('S: %.2fcm/s' % speed, 28, 30) # display speed
    disp.show()


# Clean up and exit
f.close()
kooka.radio.disable()