# File name: WarmMe.py
# Copyright: The AustSTEM Foundation Limited
# Author: Tony Strasser
# Date created: 5 January 2019
# Date last modified: 19 Jan 2020
# MicroPython Version: 1.11 for the Kookaberry V4-05
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# To the fullest extent permitted by law, AustSTEM absolutely disclaims 
# all warranties, expressed or implied, including, but not limited to, 
# implied warranties of merchantability and fitness for any particular purpose. 
# AustSTEM gives no warranty that this software will be free of errors, 
# or that defects in the software will be corrected,  
# See the GNU General Public License for more details.
#
# Measure temperature using a DS18B20 thermistor probe

# The thermistor plugs into P5
# The output relay module plugs into P2
# Begin code
# Initial conditions
import machine, kooka, fonts, time, framebuf, math
from Kapputils import config    # utility to read the configuration file
import onewire, ds18x20
import kcpu
import logger
# Other initialisations
t_plug = 'P4' # input port to use for the thermistor - change this to suit
r_plug = 'P2'    # output port for the switch module - change this to suit
threshholds = [5,15,35,45,200]
condition = ['Frz','Cold','Amb', 'Warm', 'Hot!']
t_target = 30
t_hyst = 0.97    # temperature control lower hysterisis point
t_off = t_target
t_on = t_target * t_hyst
r_status = ['Off', 'On']
sample_interval = 2    # interval at which the sensor is sampled
thermistor = ds18x20.DS18X20(onewire.OneWire(machine.Pin(t_plug)))    # create the thermistor object
# scan for devices on the bus
roms = thermistor.scan()
print('found devices:', roms)

relay = machine.Pin(r_plug, machine.Pin.OUT)    # create the digital output for the relay
kooka.radio.disable()         # turn off radio to save battery power
# Create a warmer icon
bitmap = bytearray([0x00, 0x00, 0x00, 0xC6, 0xFF, 0x7E, 0x00, 0x00, 0x00, 0x00, 0xC6, 0xFF, 0xFE, 0x38, 0x00, 0x00,
0x00, 0x86, 0xFF, 0xFE, 0x78, 0x00, 0x00, 0x08, 0x3E, 0x7F, 0x63, 0x00, 0x00, 0x00, 0x00, 0x3E,
0x7F, 0x77, 0x00, 0x00, 0x00, 0x00, 0x1E, 0x7F, 0x77, 0x00, 0x00, 0x00])

warmer = framebuf.FrameBuffer(bitmap, 22, 16, framebuf.MONO_VLSB)

disp = kooka.display
params = config('Kappconfig.txt')   # read the configuration file
dlog = logger.Dlog(params['FILE'],int(params['INTV']),'Temperature degC,Category,Target deg C,Heater')    # Create the datalogger instance
dlog.start()    # Start the datalogger
sample_timer = 0    # the sample interval timer per configuration

relay.low() # switch the relay off to begin

while not kooka.button_a.was_pressed() and kcpu.battery_ok():
    t0 = time.ticks_ms() # mark the time in ticks at the beginning of the loop
    disp.fill(0)
    disp.setfont(fonts.mono8x8)
    disp.text('WarmMe', 0, 6)
    disp.setfont(fonts.mono5x5)
    disp.text('DS18B20', 50, 6)
    disp.setfont(fonts.mono6x7)
    disp.text('A-exit', 0, 60)
    disp.text('C-Dn  D-Up', 50, 60)
    disp.setfont(fonts.mono5x5)
    disp.text('Batt %.1fV Sensor %s Heater %s' % (kcpu.battery_volts(),t_plug, r_plug), 0, 50)
# Adjust the warming threshhold using the C and D keys
    if kooka.button_c.was_pressed():
        t_target -= 1    # decrement the target temperature
        t_off = t_target
        t_on = t_target * t_hyst
    if kooka.button_d.was_pressed():
        t_target += 1    # increment the target temperature
        t_off = t_target
        t_on = t_target * t_hyst
        
# Read and process the thermocouple sensor
    disp.setfont(fonts.mono8x13)
    if (sample_timer == 0):
        thermistor.convert_temp()
        time.sleep_ms(750)
        temp = thermistor.read_temp(roms[0])    # assume sensor is first one on the onewire

# Categorise the temperature reading
        for category in range (0,len(threshholds)):
            if temp < threshholds[category] : break
# Display the reading as a percentage and the category
    disp.text('%.1f' % temp, 10, 21)
    disp.text('%s' % condition[category], 10, 34)
    disp.text('> %2d C' % t_target, 45, 21)
# Display a bar chart with the treshholds marked
    r_width = 10
    r_height = 40
    r_x = 100
    r_y = 0
    disp.rect(r_x,r_y,r_width,r_height,1)    # outline rectangle
    # Display the lower threshhold line
    t_y = int(r_y + r_height - t_off * r_height / 100)
    t_x1 = r_x - 5
    t_x2 = r_x + r_width + 5
    disp.line(t_x1 , t_y, t_x2 , t_y, 1)
    # Display the on threshhold line
    t_y = int(r_y + r_height - t_on * r_height / 100)
    t_x1 = r_x - 5
    t_x2 = r_x + r_width + 5
    disp.line(t_x1 , t_y, t_x2 , t_y, 1)
    # Display the temperature measurement histogram line
    t_y = int(r_height - temp * r_height / 100)
    t_h = int(temp * r_height / 100)
    disp.fill_rect(r_x , t_y, r_width , t_h, 1)
# Control the relay
    if temp < t_on: relay.high()
    if temp > t_off: relay.low()
    if relay.value(): disp.blit(warmer, 55, 24)
    disp.show()

# Control the sampling interval
    print(dlog.log_counter, dlog.log_write)
    dlog.update('%0.1f,%s,%d,%s\n' % (temp, condition[category],t_target,r_status[relay.value()]))
    dlog.write()
    t1 = time.ticks_ms() # mark the time in ticks after the loop work is done
    t_sleep = 1000 - time.ticks_diff(t1, t0)    #calculate how long before 1 second is up
    time.sleep_ms(t_sleep)
    sample_timer += 1
    if (sample_timer >= sample_interval):
        sample_timer = 0
# Clean up and exit
relay.low() # switch the relay off
dlog.kill()    # disables the datalogger
